import os
from json import JSONDecodeError
from pathlib import Path
from typing import List, Union, Optional, Tuple, Any, Dict, Set, Callable, TYPE_CHECKING, AbstractSet, \
    Mapping

from httpx import Cookies
from loguru import logger
from pydantic import BaseModel, Extra, ValidationError, BaseSettings, validator

from mys_goods_tool.data_model import BaseModelWithSetter, Good, Address, GameRecord, BaseModelWithUpdate

ROOT_PATH = Path("./")
"""程序所在目录"""

CONFIG_PATH = ROOT_PATH / "user_data.json"
"""用户数据文件默认路径"""

VERSION = "2.1.0-beta.1"
"""程序当前版本"""

if TYPE_CHECKING:
    IntStr = Union[int, str]
    DictStrAny = Dict[str, Any]
    AbstractSetIntStr = AbstractSet[IntStr]
    MappingIntStrAny = Mapping[IntStr, Any]


class BBSCookies(BaseModelWithSetter, BaseModelWithUpdate):
    """
    米游社Cookies数据

    # 测试 is_correct() 方法

    >>> assert BBSCookies().is_correct() is False
    >>> assert BBSCookies(stuid="123", stoken="123", cookie_token="123").is_correct() is True

    # 测试 bbs_uid getter

    >>> bbs_cookies = BBSCookies()
    >>> assert not bbs_cookies.bbs_uid
    >>> assert BBSCookies(stuid="123").bbs_uid == "123"

    # 测试 bbs_uid setter

    >>> bbs_cookies.bbs_uid = "123"
    >>> assert bbs_cookies.bbs_uid == "123"

    # 检查构造函数内所用的 stoken setter

    >>> bbs_cookies = BBSCookies(stoken="abcd1234")
    >>> assert bbs_cookies.stoken_v1 and not bbs_cookies.stoken_v2
    >>> bbs_cookies = BBSCookies(stoken="v2_abcd1234==")
    >>> assert bbs_cookies.stoken_v2 and not bbs_cookies.stoken_v1
    >>> assert bbs_cookies.stoken == "v2_abcd1234=="

    # 检查 stoken setter

    >>> bbs_cookies = BBSCookies(stoken="abcd1234")
    >>> bbs_cookies.stoken = "v2_abcd1234=="
    >>> assert bbs_cookies.stoken_v2 == "v2_abcd1234=="
    >>> assert bbs_cookies.stoken_v1 == "abcd1234"

    # 检查 .dict 方法能否生成包含 stoken_2 类型的 stoken 的字典

    >>> bbs_cookies = BBSCookies()
    >>> bbs_cookies.stoken_v1 = "abcd1234"
    >>> bbs_cookies.stoken_v2 = "v2_abcd1234=="
    >>> assert bbs_cookies.dict(v2_stoken=True)["stoken"] == "v2_abcd1234=="

    # 检查是否有多余的字段

    >>> bbs_cookies = BBSCookies(stuid="123")
    >>> assert all(bbs_cookies.dict())
    >>> assert all(map(lambda x: x not in bbs_cookies, ["stoken_v1", "stoken_v2"]))

    # 测试 update 方法

    >>> bbs_cookies = BBSCookies(stuid="123")
    >>> assert bbs_cookies.update({"stuid": "456", "stoken": "abc"}) is bbs_cookies
    >>> assert bbs_cookies.stuid == "456"
    >>> assert bbs_cookies.stoken == "abc"

    >>> bbs_cookies = BBSCookies(stuid="123")
    >>> new_cookies = BBSCookies(stuid="456", stoken="abc")
    >>> assert bbs_cookies.update(new_cookies) is bbs_cookies
    >>> assert bbs_cookies.stuid == "456"
    >>> assert bbs_cookies.stoken == "abc"
    """
    stuid: Optional[str]
    """米游社UID"""
    ltuid: Optional[str]
    """米游社UID"""
    account_id: Optional[str]
    """米游社UID"""
    login_uid: Optional[str]
    """米游社UID"""

    stoken_v1: Optional[str]
    """保存stoken_v1，方便后续使用"""
    stoken_v2: Optional[str]
    """保存stoken_v2，方便后续使用"""

    cookie_token: Optional[str]
    login_ticket: Optional[str]
    ltoken: Optional[str]
    mid: Optional[str]

    def __init__(self, **data: Any):
        super().__init__(**data)
        stoken = data.get("stoken")
        if stoken:
            self.stoken = stoken

    def is_correct(self) -> bool:
        """判断是否为正确的Cookies"""
        if self.bbs_uid and self.stoken and self.cookie_token:
            return True
        else:
            return False

    @property
    def bbs_uid(self):
        """
        获取米游社UID
        """
        uid = None
        for value in [self.stuid, self.ltuid, self.account_id, self.login_uid]:
            if value:
                uid = value
                break
        return uid or None

    @bbs_uid.setter
    def bbs_uid(self, value: str):
        self.stuid = value
        self.ltuid = value
        self.account_id = value
        self.login_uid = value

    @property
    def stoken(self):
        """
        获取stoken
        :return: 优先返回 self.stoken_v1
        """
        if self.stoken_v1:
            return self.stoken_v1
        elif self.stoken_v2:
            return self.stoken_v2
        else:
            return None

    @stoken.setter
    def stoken(self, value):
        if value.startswith("v2_"):
            self.stoken_v2 = value
        else:
            self.stoken_v1 = value

    def update(self, cookies: Union[Dict[str, str], Cookies, "BBSCookies"]):
        """
        更新Cookies
        """
        if not isinstance(cookies, BBSCookies):
            self.stoken = cookies.get("stoken") or self.stoken
            self.bbs_uid = cookies.get("bbs_uid") or self.bbs_uid
            cookies.pop("stoken", None)
            cookies.pop("bbs_uid", None)
        return super().update(cookies)

    def dict(self, *,
             include: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
             exclude: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
             by_alias: bool = False,
             skip_defaults: Optional[bool] = None, exclude_unset: bool = False, exclude_defaults: bool = False,
             exclude_none: bool = False, v2_stoken: bool = False,
             cookie_type: bool = False) -> 'DictStrAny':
        """
        获取Cookies字典

        v2_stoken: stoken 字段是否使用 stoken_v2
        cookie_type: 是否返回符合Cookie类型的字典（没有自定义的stoken_v1、stoken_v2键）
        """
        # 保证 stuid, ltuid 等字段存在
        self.bbs_uid = self.bbs_uid
        cookies_dict = super().dict(include=include, exclude=exclude, by_alias=by_alias, skip_defaults=skip_defaults,
                                    exclude_unset=exclude_unset, exclude_defaults=exclude_defaults,
                                    exclude_none=exclude_none)
        if v2_stoken:
            cookies_dict["stoken"] = self.stoken_v2

        if cookie_type:
            # 去除自定义的 stoken_v1, stoken_v2 字段
            cookies_dict.pop("stoken_v1")
            cookies_dict.pop("stoken_v2")

            # 去除空的字段
            empty_key = set()
            for key, value in cookies_dict.items():
                if not value:
                    empty_key.add(key)
            [cookies_dict.pop(key) for key in empty_key]

        return cookies_dict


class UserAccount(BaseModelWithSetter, extra=Extra.ignore):
    """
    米游社账户数据

    >>> user_account = UserAccount(cookies=BBSCookies())
    >>> assert isinstance(user_account, UserAccount)
    >>> user_account.bbs_uid = "123"
    >>> assert user_account.bbs_uid == "123"
    """
    phone_number: Optional[str]
    """手机号"""
    cookies: BBSCookies
    """Cookies"""

    device_id_ios: str
    """iOS设备用 deviceID"""
    device_id_android: str
    """安卓设备用 deviceID"""
    device_fp: Optional[str]
    """iOS设备用 deviceFp"""

    def __init__(self, **data: Any):
        if not data.get("device_id_ios") or not data.get("device_id_android"):
            from mys_goods_tool.utils import generate_device_id
            if not data.get("device_id_ios"):
                data.setdefault("device_id_ios", generate_device_id())
            if not data.get("device_id_android"):
                data.setdefault("device_id_android", generate_device_id())
        super().__init__(**data)

    @property
    def bbs_uid(self):
        """
        获取米游社UID
        """
        return self.cookies.bbs_uid

    @bbs_uid.setter
    def bbs_uid(self, value: str):
        self.cookies.bbs_uid = value


class ExchangePlan(BaseModel):
    """
    兑换计划数据类
    """
    good: Good
    """商品"""
    address: Optional[Address]
    """地址ID"""
    account: UserAccount
    """米游社账号"""
    game_record: Optional[GameRecord]
    """商品对应的游戏的玩家账号"""

    def __hash__(self):
        return hash(
            (
                self.good.goods_id,
                self.good.time,
                self.address.id if self.address else None,
                self.account.bbs_uid,
                self.game_record.game_role_id if self.game_record else None
            )
        )


class ExchangeResult(BaseModel):
    """
    兑换结果数据类
    """
    result: bool
    """兑换结果"""
    return_data: dict
    """返回数据"""
    plan: ExchangePlan
    """兑换计划"""


class Preference(BaseSettings):
    """
    偏好设置
    """
    github_proxy: Optional[str] = "https://ghproxy.com/"
    """GitHub加速代理 最终会拼接在原GitHub链接前面"""
    enable_connection_test: bool = True
    """是否开启连接测试"""
    connection_test_interval: Optional[float] = 30
    """连接测试间隔（单位：秒）"""
    timeout: float = 10
    """网络请求超时时间（单位：秒）"""
    max_retry_times: Optional[int] = 3
    """最大网络请求重试次数"""
    retry_interval: float = 2
    """网络请求重试间隔（单位：秒）（除兑换请求外）"""
    enable_ntp_sync: Optional[bool] = True
    """是否开启NTP时间同步（将调整实际发出兑换请求的时间，而不是修改系统时间）"""
    ntp_server: Optional[str] = "ntp.aliyun.com"
    """NTP服务器地址"""
    timezone: Optional[str] = "Asia/Shanghai"
    """兑换时所用的时区"""
    geetest_statics_path: Optional[Path]
    """GEETEST行为验证 网站静态文件目录（默认读取本地包自带的静态文件）"""
    geetest_listen_address: Optional[Tuple[str, int]] = ("localhost", 0)
    """登录时使用的 GEETEST行为验证 WEB服务 本地监听地址"""
    exchange_thread_count: int = 2
    """兑换线程数"""
    exchange_latency: Tuple[float, float] = (0, 0.2)
    """兑换时间延迟随机范围（单位：秒）（防止因为发出请求的时间过于精准而被服务器认定为非人工操作）"""
    enable_log_output: bool = True
    """是否保存日志"""
    log_path: Optional[Path] = ROOT_PATH / "logs" / "mys_goods_tool.log"
    """日志保存路径"""
    override_device_and_salt: bool = False
    """是否读取用户数据文件中的 device_config 设备配置 和 salt_config 配置而不是默认配置（一般情况不建议开启）"""

    @validator("log_path")
    def _(cls, v: Optional[Path]):
        absolute_path = v.absolute()
        if not os.path.exists(absolute_path) or not os.path.isfile(absolute_path):
            absolute_parent = absolute_path.parent
            try:
                os.makedirs(absolute_parent, exist_ok=True)
            except PermissionError:
                logger.warning(f"程序没有创建日志目录 {absolute_parent} 的权限")
        elif not os.access(absolute_path, os.W_OK):
            logger.warning(f"程序没有写入日志文件 {absolute_path} 的权限")
        return v

    class Config:
        env_prefix = "MYS_GOODS_TOOL_"  # 环境变量前缀


class SaltConfig(BaseSettings):
    """
    生成Headers - DS所用salt值，非必要请勿修改
    """
    SALT_IOS: str = "ulInCDohgEs557j0VsPDYnQaaz6KJcv5"
    '''生成Headers iOS DS所需的salt'''
    SALT_ANDROID: str = "n0KjuIrKgLHh08LWSCYP0WXlVXaYvV64"
    '''生成Headers Android DS所需的salt'''
    SALT_DATA: str = "t0qEgfub6cvueAPgR5m9aQWWVciEer7v"
    '''Android 设备传入content生成 DS 所需的 salt'''
    SALT_PARAMS: str = "xV8v4Qu54lUKrEYFZkJhB8cuOh9Asafs"
    '''Android 设备传入url参数生成 DS 所需的 salt'''
    SALT_PROD: str = "JwYDpKvLj6MrMqqYU6jTKF17KNO2PXoS"

    class Config(Preference.Config):
        pass


# TODO 与用户数据文件中的配置有差异时询问是否使用更新
class DeviceConfig(BaseSettings):
    """
    设备信息
    Headers所用的各种数据，非必要请勿修改
    """
    USER_AGENT_MOBILE: str = "Mozilla/5.0 (iPhone; CPU iPhone OS 15_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) miHoYoBBS/2.54.1"
    '''移动端 User-Agent(Mozilla UA)'''
    USER_AGENT_PC: str = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.0 Safari/605.1.15"
    '''桌面端 User-Agent(Mozilla UA)'''
    USER_AGENT_OTHER: str = "Hyperion/275 CFNetwork/1402.0.8 Darwin/22.2.0"
    '''获取用户 ActionTicket 时Headers所用的 User-Agent'''
    USER_AGENT_ANDROID: str = "Mozilla/5.0 (Linux; Android 11; MI 8 SE Build/RQ3A.211001.001; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/104.0.5112.97 Mobile Safari/537.36 miHoYoBBS/2.54.1"
    '''安卓端 User-Agent(Mozilla UA)'''
    USER_AGENT_ANDROID_OTHER: str = "okhttp/4.9.3"
    '''安卓端 User-Agent(专用于米游币任务等)'''
    USER_AGENT_WIDGET: str = "WidgetExtension/231 CFNetwork/1390 Darwin/22.0.0"
    '''iOS 小组件 User-Agent(原神实时便笺)'''

    X_RPC_DEVICE_MODEL_MOBILE: str = "iPhone10,2"
    '''移动端 x-rpc-device_model'''
    X_RPC_DEVICE_MODEL_PC: str = "OS X 10.15.7"
    '''桌面端 x-rpc-device_model'''
    X_RPC_DEVICE_MODEL_ANDROID: str = "MI 8 SE"
    '''安卓端 x-rpc-device_model'''

    X_RPC_DEVICE_NAME_MOBILE: str = "iPhone"
    '''移动端 x-rpc-device_name'''
    X_RPC_DEVICE_NAME_PC: str = "Microsoft Edge 103.0.1264.62"
    '''桌面端 x-rpc-device_name'''
    X_RPC_DEVICE_NAME_ANDROID: str = "Xiaomi MI 8 SE"
    '''安卓端 x-rpc-device_name'''

    X_RPC_SYS_VERSION: str = "15.4"
    '''Headers所用的 x-rpc-sys_version'''
    X_RPC_SYS_VERSION_ANDROID: str = "11"
    '''安卓端 x-rpc-sys_version'''

    X_RPC_CHANNEL: str = "appstore"
    '''Headers所用的 x-rpc-channel'''
    X_RPC_CHANNEL_ANDROID: str = "miyousheluodi"
    '''安卓端 x-rpc-channel'''

    X_RPC_APP_VERSION: str = "2.54.1"
    '''Headers所用的 x-rpc-app_version'''
    X_RPC_PLATFORM: str = "ios"
    '''Headers所用的 x-rpc-platform'''
    UA: str = "\".Not/A)Brand\";v=\"99\", \"Microsoft Edge\";v=\"103\", \"Chromium\";v=\"103\""
    '''Headers所用的 sec-ch-ua'''
    UA_PLATFORM: str = "\"macOS\""
    '''Headers所用的 sec-ch-ua-platform'''

    class Config(Preference.Config):
        pass


class UserData(BaseModel):
    """
    用户数据类
    """
    version: str = VERSION
    """创建用户数据文件的程序版本号"""
    exchange_plans: Union[Set[ExchangePlan], List[ExchangePlan]] = set()
    """兑换计划列表"""
    preference: Preference = Preference()
    """偏好设置"""
    salt_config: SaltConfig = SaltConfig()
    """生成Headers - DS所用salt值"""
    device_config: DeviceConfig = DeviceConfig()
    """设备信息"""
    accounts: Dict[str, UserAccount] = {}
    """储存一些已绑定的账号数据"""

    def __init__(self, **data: Any):
        super().__init__(**data)
        exchange_plans = self.exchange_plans
        self.exchange_plans = set()
        for plan in exchange_plans:
            plan = ExchangePlan.parse_obj(plan)
            self.exchange_plans.add(plan)

    def save(self):
        """
        保存用户数据文件
        """
        return write_config_file(self)

    def json(
            self,
            *,
            include: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
            exclude: Optional[Union['AbstractSetIntStr', 'MappingIntStrAny']] = None,
            by_alias: bool = False,
            skip_defaults: Optional[bool] = None,
            exclude_unset: bool = False,
            exclude_defaults: bool = False,
            exclude_none: bool = False,
            encoder: Optional[Callable[[Any], Any]] = None,
            models_as_dict: bool = True,
            **dumps_kwargs: Any,
    ) -> str:
        """
        重写 BaseModel.json() 方法，使其支持对 Set 类型的数据进行序列化
        """
        set_plans = self.exchange_plans
        self.exchange_plans = list(set_plans)
        json_data = super().json(
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            skip_defaults=skip_defaults,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
            encoder=encoder,
            models_as_dict=models_as_dict,
            **dumps_kwargs,
        )
        self.exchange_plans = set_plans
        return json_data


def write_config_file(conf: UserData = UserData()):
    """
    写入用户数据文件

    :param conf: 配置对象
    """
    try:
        str_data = conf.json(indent=4)
    except (AttributeError, TypeError, ValueError):
        logger.exception("数据对象序列化失败，可能是数据类型错误")
        return False
    with open(CONFIG_PATH, "w", encoding="utf-8") as f:
        f.write(str_data)
    return True


def load_config() -> Tuple[UserData, bool]:
    """
    加载用户数据文件

    :return: (<用户数据对象>, <device_config 或 salt_config 是否非默认值且未开启覆写>)
    """
    if os.path.exists(CONFIG_PATH) and os.path.isfile(CONFIG_PATH):
        try:
            user_data = UserData.parse_file(CONFIG_PATH)
        except (ValidationError, JSONDecodeError):
            logger.exception(f"读取用户数据文件失败，请检查用户数据文件 {CONFIG_PATH} 格式是否正确")
            exit(1)
        except:
            logger.exception(f"读取用户数据文件失败，请检查用户数据文件 {CONFIG_PATH} 是否存在且程序有权限读取和写入")
            exit(1)
        else:
            if not user_data.preference.override_device_and_salt:
                default_device_config = DeviceConfig()
                default_salt_config = SaltConfig()
                if user_data.device_config != default_device_config or user_data.salt_config != default_salt_config:
                    user_data.device_config = default_device_config
                    user_data.salt_config = default_salt_config
                    return user_data, True
            else:
                logger.info("已开启覆写 device_config 和 salt_config，将读取用户数据文件中的配置以覆写默认配置")
            return user_data, False
    else:
        user_data = UserData()
        try:
            write_config_file(user_data)
        except PermissionError:
            logger.exception(f"创建用户数据文件失败，请检查程序是否有权限读取和写入 {CONFIG_PATH}")
            exit(1)
        # logger.info(f"用户数据文件 {CONFIG_PATH} 不存在，已创建默认用户数据文件。")
        # 由于会输出到标准输出流，影响TUI观感，因此暂时取消

        return user_data, False


_load_config_return = load_config()
config = _load_config_return[0]
"""程序配置对象"""
different_device_and_salt = _load_config_return[1]
"""device_config 或 salt_config 是否非默认值且未开启覆写"""



from __future__ import annotations

from abc import abstractmethod
from typing import Tuple, Optional, List, Dict, Union, Type, TypeVar

from rich.console import RenderableType
from rich.markdown import Markdown
from textual import events
from textual.app import ComposeResult
from textual.reactive import reactive
from textual.widget import Widget
from textual.widgets import (
    TabbedContent, TabPane, OptionList, ListView
)
from textual.widgets._option_list import Option, Separator

from mys_goods_tool.api import get_good_list, get_address, get_game_record, good_exchange, \
    get_good_detail, get_good_games, get_device_fp
from mys_goods_tool.custom_css import *
from mys_goods_tool.custom_widget import StaticStatus, ControllableButton, LoadingDisplay, \
    DynamicTabbedContent, GameButton, PlanButton, UnClickableItem
from mys_goods_tool.data_model import Good, Address, GameRecord
from mys_goods_tool.user_data import config as conf, UserAccount, ExchangePlan
from mys_goods_tool.utils import logger

_T = TypeVar("_T")


class BaseExchangePlan(ExchangePlanContent):
    DEFAULT_TEXT: RenderableType
    """默认提示文本内容"""
    text_view: StaticStatus
    """实时文本提示"""

    button_select: ControllableButton
    """保存选定内容"""
    button_refresh: ControllableButton
    """刷新列表"""
    button_reset: ControllableButton
    """重置选择"""
    _selected: Optional[Type[_T]] = None
    """已选内容"""

    empty_data_option: Option
    """可选列表为空时显示的视图"""

    @property
    def selected(self):
        """已选内容"""
        return self._selected

    @selected.setter
    def selected(self, value: Type[_T]):
        """设置已选内容的同时更新CheckOutText兑换计划预览视图"""
        type(self)._selected = value
        FinishContent.check_out_text.set_check_item(value, type(self))

    @abstractmethod
    def reset_selected(self):
        """
        重置已选内容
        一般包含以下操作：
            - 清空已选内容
            - 禁用重置按钮
            - 启用选择按钮
            - 重置文本内容
        """
        pass

    @abstractmethod
    def _on_button_pressed(self, event: ControllableButton.Pressed) -> None:
        """
        按下按钮时触发的事件
        一般包含：
            - 选择按钮：保存选定内容
                - 禁用选择按钮
                - 禁用选项列表
                - 启用重置按钮
                - 更新文本内容
            - 刷新按钮：刷新列表
                - 更新选项列表
                - 检查新的列表是否为空
                - 重置已选内容
            - 重置按钮：重置已选内容
                - 重置已选内容
                - 重置文本内容
                - 禁用重置按钮
                - 启用选择按钮
                - 启用选项列表
        """
        pass

    @abstractmethod
    def update_data(self):
        """
        更新数据
        一般包含：
            - 更新选项列表
            - 如果列表为空，加入空数据提示项
            - 重置已选内容
        """
        pass

    def reset_all(self) -> None:
        """
        重置所有内容
        - 重置已选游戏账号
        - 重置文本内容
        - 禁用所有按钮
        - 禁用选项列表
        - 清空选项列表
        """
        pass


class AccountContent(BaseExchangePlan):
    """
    选择账号视图
    """
    DEFAULT_TEXT = Markdown("- 请选择一个米游社账号")
    text_view = StaticStatus(DEFAULT_TEXT)

    button_select = ControllableButton("💾 保存", id="button-account-select", disabled=True)
    button_refresh = ControllableButton("🔄 刷新", variant="primary", id="button-account-refresh")
    button_reset = ControllableButton("↩ 重置", variant="warning", id="button-account-reset", disabled=True)

    account_keys = list(conf.accounts.keys())
    """账号列表"""
    option_list = OptionList(*account_keys, disabled=True)
    """账号选项列表"""
    empty_data_option = Option("暂无米游社账号数据 请尝试刷新", disabled=True)

    if account_keys:
        # 如果账号列表非空，启用 选择按钮、选项列表
        button_select.enable()
        option_list.disabled = False
    else:
        option_list.add_option(empty_data_option)

    def __init__(self, *children: Widget):
        super().__init__(*children)

    def compose(self) -> ComposeResult:
        yield self.text_view
        with Horizontal():
            yield self.button_select
            yield self.button_refresh
            yield self.button_reset
        yield self.option_list

    def reset_selected(self):
        """
        重置账号选择
        """
        # 刷新选项列表后检查是否为空
        if not self.account_keys:
            self.option_list.disabled = True
            self.button_select.disable()
        else:
            self.option_list.disabled = False
            self.button_select.enable()

        self.selected = None
        self.button_reset.disable()
        self.text_view.update(self.DEFAULT_TEXT)
        ExchangePlanView.address_content.reset_all()
        ExchangePlanView.game_record_content.reset_all()

    def update_data(self):
        """
        更新账号列表
        """
        self.account_keys = list(conf.accounts.keys())
        self.option_list.clear_options()
        for account in self.account_keys:
            self.option_list.add_option(account)
        if not self.account_keys:
            self.option_list.add_option(self.empty_data_option)
        # 重置已选内容
        self.reset_selected()

    async def _on_button_pressed(self, event: ControllableButton.Pressed) -> None:
        if event.button.id == "button-account-select":
            # 按下“保存”按钮时触发的事件

            if self.option_list.highlighted is None:
                self.app.notice("[bold red]请先从列表中选择米游社账号！[/]")
                return

            account_key = self.account_keys[self.option_list.highlighted]
            self.selected = conf.accounts.get(account_key)
            if self.selected is None:
                self.app.notice(f"未找到米游社账号：[bold red]{account_key}[/]")
                return

            # 禁用选择按钮、启用重置按钮、禁用选项列表
            self.button_select.disable()
            self.button_reset.enable()
            self.option_list.disabled = True

            self.text_view.update(f"已选择米游社账号"
                                  f"\n[list]"
                                  f"\n🪪 通信证ID - [bold green]{account_key}[/]"
                                  f"\n[/list]")
            if conf.accounts[account_key].cookies.is_correct():
                self.app.notice(f"选择的米游社账号：[bold green]{account_key}[/] Cookies完整，可继续")
            else:
                self.app.notice(
                    f"选择的米游社账号：[bold red]{account_key}[/] Cookies不完整，但你仍然可以尝试进行兑换")

            await ExchangePlanView.address_content.update_data()
            await ExchangePlanView.game_record_content.update_data()

        elif event.button.id == "button-account-refresh":
            # 按下“刷新”按钮时触发的事件

            self.update_data()
            self.app.notice("[bold green]已刷新米游社账号列表[/]")

        elif event.button.id == "button-account-reset":
            # 按下“重置”按钮时触发的事件

            self.reset_selected()
            self.app.notice("已重置米游社账号选择")


class GoodsContent(BaseExchangePlan):
    """
    选择商品视图
    """
    DEFAULT_CSS = """
    GoodsWidget TabbedContainer {
        height: 100%;
        width: 100%;
    }
    """
    DEFAULT_TEXT = Markdown("- 请选择一个商品")
    text_view = StaticStatus(DEFAULT_TEXT)

    button_refresh = ControllableButton("🔄 刷新", variant="primary", id="button-goods-refresh")
    button_reset = ControllableButton("↩ 重置", variant="warning", id="button-goods-reset", disabled=True)

    loading = LoadingDisplay()
    loading.hide()

    good_dict: Dict[str, GoodsDictValue] = {}
    """获取到的商品数据以及相关的控件 商品分区简称 -> 商品数据"""
    selected_tuple: Optional[Tuple[Tuple[str, str], int]] = None
    """已选择的商品位置 ((商品分区, 分区简称), 商品在OptionList中的位置)"""

    empty_data_option = Option("暂无商品数据，可能是目前没有限时兑换的商品，可尝试刷新", disabled=True)
    """空的商品选项列表"""
    tabbed_content = DynamicTabbedContent()

    class GoodsDictValue:
        """
        游戏频道对应的商品数据相关
        """

        def __init__(self,
                     partition: Tuple[str, str],
                     button_select: Optional[GameButton] = None,
                     tap_pane: Optional[TabPane] = None,
                     good_list: List[Good] = None,
                     ):
            """
            :param partition: (商品分区, 字母简称) 数据
            :param tap_pane: 频道对应的 `TabPane` 标签页
            :param good_list: 商品数据
            :param button_select: 选择商品的按钮
            """
            name, abbr = partition
            self.partition = partition
            """商品频道数据"""
            self.button_select = button_select or GameButton(
                "💾 确定",
                id=f"button-goods-select-{abbr}",
                disabled=True,
                partition=partition)
            """选择商品的按钮"""
            self.option_list = OptionList(GoodsContent.empty_data_option, disabled=True)
            """商品的选项列表"""
            self.tap_pane = tap_pane or TabPane(name, Horizontal(self.button_select, self.option_list))
            """频道对应的 `TabPane` 标签页"""
            self.good_list = good_list
            """商品数据"""

    def compose(self) -> ComposeResult:
        yield self.text_view
        with Horizontal():
            yield self.button_refresh
            yield self.button_reset
            yield self.loading
        yield self.tabbed_content

    async def update_data(self):
        """
        刷新商品信息
        """
        # 进度条、刷新按钮
        self.loading.show()
        self.button_refresh.disable()

        for goods_data in self.good_dict.values():
            name, abbr = goods_data.partition
            good_list_status, good_list = await get_good_list(abbr)
            good_list = list(filter(lambda x: x.time_limited and not x.time_end, good_list))

            # 一种情况是获取成功但返回的商品数据为空，一种是API请求失败
            goods_data.option_list.clear_options()
            if not good_list_status:
                self.app.notice(f"[bold red]获取频道 [bold red]{name}[/] 的商品数据失败！[/]")
                # TODO 待补充各种错误情况
            if good_list:
                goods_data.good_list = good_list
                good_names = map(lambda x: x.general_name, good_list)
                for name in good_names:
                    goods_data.option_list.add_option(name)
                goods_data.button_select.enable()
                goods_data.option_list.disabled = False
            else:
                goods_data.option_list.add_option(self.empty_data_option)

        # 进度条、刷新按钮
        self.loading.hide()
        self.button_refresh.enable()

        # 重置已选内容（包含启用 选择按钮、选项列表）
        self.reset_selected()

    async def _on_mount(self, _: events.Mount):
        # 进度条、刷新按钮
        self.button_refresh.disable()
        self.loading.show()

        # 更新商品频道列表
        partition_status, partition_all = await get_good_games()

        # 刷新按钮、进度条
        self.button_refresh.enable()
        self.loading.hide()

        if partition_status:
            # 过滤掉 "全部" 分区
            partitions = filter(lambda x: x[1] != "all", partition_all)
            for name, abbr in partitions:
                if abbr not in self.good_dict:
                    # 如果没有商品频道对应值，则进行创建
                    goods_data = self.GoodsDictValue((name, abbr))
                    self.good_dict.setdefault(abbr, goods_data)
                    await self.tabbed_content.append(goods_data.tap_pane)

            # 更新每个频道的商品数据
            await self.update_data()
            return True
        else:
            self.text_view.update("[bold red]⚠ 获取商品频道列表失败，可尝试刷新[/]")
            self.app.notice("[bold red]获取商品频道列表失败！[/]")
            # TODO 待补充各种错误情况
            return False

    def reset_selected(self):
        """
        重置商品选择
        """
        # 刷新选项列表后检查是否为空
        for value in self.good_dict.values():
            if value.good_list:
                value.button_select.enable()
                value.option_list.disabled = False
            else:
                value.button_select.disable()
                value.option_list.disabled = True

        self.selected = None
        self.button_reset.disable()
        self.selected_tuple = None
        self.text_view.update(self.DEFAULT_TEXT)

        AddressContent.check_good_type()
        ExchangePlanView.game_record_content.reset_selected()
        GameRecordContent.check_good_type()

    async def _on_button_pressed(self, event: GameButton.Pressed) -> None:
        if event.button.id.startswith("button-goods-select"):
            # 按下“保存”按钮时触发的事件

            name, abbr = event.button.partition
            option_list = self.good_dict[abbr].option_list
            selected_index = option_list.highlighted
            if selected_index is None:
                self.app.notice("[bold red]请先从列表中选择商品！[/]")
                return
            good_dict_value = self.good_dict.get(abbr)
            if not good_dict_value:
                self.app.notice("[bold red]未找到对应的频道[/]")
                return

            good = good_dict_value.good_list[selected_index]
            GoodsContent.selected_tuple = name, abbr, selected_index

            # 获取商品详情
            self.loading.show()
            good_detail_status, _ = await get_good_detail(good)
            self.loading.hide()
            if not good_detail_status:
                # TODO 待补充各种错误情况
                self.app.notice("[bold red]获取商品详情(game_biz)失败，但你仍然可以尝试进行兑换[/]")
            self.selected = good

            # 启用重置按钮
            self.button_reset.enable()

            # 禁用其他频道的选择按钮
            # 禁用其他频道的选项列表
            for value in self.good_dict.values():
                value.button_select.disable()
                value.option_list.disabled = True

            # 如果是虚拟/实物商品，则地址、游戏账号视图需要更新
            AddressContent.check_good_type()
            if AccountContent._selected is not None:
                GameRecordContent.check_good_type()

            self.text_view.update(f"已选择商品："
                                  f"\n[list]"
                                  f"\n🗂️ 商品频道：[bold green]{name}[/]"
                                  f"\n📌 名称：[bold green]{good.general_name}[/]"
                                  f"\n💰 价格：[bold green]{good.price}[/] 米游币"
                                  f"\n📦 库存：[bold green]{good.stoke_text}[/] 件"
                                  f"\n📅 兑换时间：[bold green]{good.time_text}[/]"
                                  f"\n📌 商品ID：[bold green]{good.goods_id}[/]"
                                  f"\n[/list]")

            if good.is_virtual and AccountContent._selected is not None:
                await ExchangePlanView.game_record_content.update_data()

        elif event.button.id == "button-goods-refresh":
            # 按下“刷新”按钮时触发的事件

            # 在初次加载时，如果获取商品频道信息失败，则此时重新获取
            if not self.good_dict:
                good_games_result = await self._on_mount(events.Mount())
            else:
                good_games_result = True
            if good_games_result:
                await self.update_data()

        elif event.button.id == "button-goods-reset":
            # 按下“重置”按钮时触发的事件

            self.reset_selected()
            self.app.notice("已重置商品选择")


class GameRecordContent(BaseExchangePlan):
    """
    游戏账号选择视图
    """
    DEFAULT_TEXT = Markdown("- 请选择一个游戏账号")
    REQUIRE_OTHER_TEXT = Markdown("- 请先完成米游社账号和商品选择")
    UNNEEDED_TEXT = Markdown("- 兑换实体商品无需选择游戏账号")

    text_view = StaticStatus(REQUIRE_OTHER_TEXT)

    button_select = ControllableButton("💾 保存", id="button-game_uid-select", disabled=True)
    button_refresh = ControllableButton("🔄 刷新", variant="primary", id="button-game_uid-refresh", disabled=True)
    button_reset = ControllableButton("↩ 重置", variant="warning", id="button-game_uid-reset", disabled=True)

    loading = LoadingDisplay()
    loading.hide()

    empty_data_option = Option("无可用账号", disabled=True)
    option_list = OptionList(REQUIRE_OTHER_TEXT, disabled=True)
    """游戏账号选项列表"""
    option_list.highlighted = None
    record_list: List[GameRecord] = []
    """游戏账号列表"""

    async def update_data(self):
        """
        更新游戏账号列表
        """
        if GoodsContent._selected is None:
            return

        # 进度条、刷新按钮、选项列表
        self.loading.show()
        self.button_refresh.disable()
        self.option_list.disabled = False

        record_status, GameRecordContent.record_list = await get_game_record(AccountContent._selected)
        self.option_list.clear_options()
        if not record_status:
            self.app.notice(f"[bold red]获取游戏账号列表失败！[/]")
        if self.record_list:
            self.option_list.add_option(Separator())
            for record in self.record_list:
                preview_text = f"[list]" \
                               f"\n👓 昵称：[bold underline]{record.nickname}[/]" \
                               f"\n📌 游戏UID：[bold underline]{record.game_role_id}[/]" \
                               f"\n🌐 区服：[bold underline]{record.region_name}[/]" \
                               f"\n🔥 等级：[bold underline]{record.level}[/]" \
                               f"\n[/list]"
                self.option_list.add_option(Option(preview_text))
                self.option_list.add_option(Separator())
                self.option_list.disabled = False
                self.button_select.enable()
        else:
            self.option_list.add_option(self.empty_data_option)

        # 进度条、刷新按钮
        self.loading.hide()
        self.button_refresh.enable()

        #  重置已选择的游戏账号
        self.reset_selected()

        # 检查选项列表是否为空的操作包含在 check_good_type 中
        self.check_good_type()

    def reset_all(self):
        self.selected = None
        self.text_view.update(self.REQUIRE_OTHER_TEXT)
        self.option_list.disabled = True
        self.option_list.clear_options()
        self.option_list.add_option(self.REQUIRE_OTHER_TEXT)
        self.button_select.disable()
        self.button_reset.disable()
        self.button_refresh.disable()

    @classmethod
    def check_empty(cls):
        """
        检查选项列表是否为空
        """
        if cls.record_list:
            cls.button_select.enable()
            cls.option_list.disabled = False
        else:
            cls.button_select.disable()
            cls.option_list.disabled = True

    @classmethod
    def check_good_type(cls):
        """
        检查商品类型是否是虚拟商品，并改变视图
        """
        # 程序载入初次刷新商品列表时，重置已选商品并调用check_good_type，由于没有选择商品，不需要检查商品类型
        if GoodsContent._selected is not None:
            good: Optional[Good] = GoodsContent._selected
            if good is not None and not good.is_virtual:
                cls.text_view.update(cls.UNNEEDED_TEXT)
                cls.option_list.disabled = True
                cls.button_select.disable()
                cls.button_refresh.disable()
            elif cls._selected is None:
                cls.text_view.update(cls.DEFAULT_TEXT)
                cls.check_empty()
                cls.button_refresh.enable()
            else:
                # 在已选游戏账号不为空的情况下，视图被虚拟商品改变后的情况
                ExchangePlanView.game_record_content._set_select_view(cls._selected)
        else:
            # # 程序载入初次刷新商品列表时，重置已选商品并调用本类的reset_selected，由于没有选择商品，需要更新文本视图
            cls.text_view.update(cls.REQUIRE_OTHER_TEXT)

    def reset_selected(self):
        self.selected = None
        self.button_reset.disable()
        self.text_view.update(self.DEFAULT_TEXT)
        self.check_good_type()

    def _set_select_view(self, record: GameRecord):
        """
        设置已选地址后改变视图
        """
        self.text_view.update(f"已选择游戏账号："
                              f"\n[list]"
                              f"\n📌 游戏UID - [bold green]{record.game_role_id}[/]"
                              f"\n[/list]")

        # 禁用 选项列表、保存按钮，启用 重置按钮
        self.button_reset.enable()
        self.button_select.disable()
        self.option_list.disabled = True

    async def _on_button_pressed(self, event: ControllableButton.Pressed) -> None:
        if event.button.id == "button-game_uid-select":
            # 按下“保存”按钮时触发的事件

            record_index = self.option_list.highlighted
            if record_index is None:
                self.app.notice("[bold red]请先从列表中选择游戏账号！[/]")
                return
            if record_index >= len(self.record_list):
                self.app.notice("[bold red]无法找到游戏账号[/]")
                return
            record = self.record_list[record_index]
            self.selected = record

            self._set_select_view(record)

        elif event.button.id == "button-game_uid-refresh":
            # 按下“刷新”按钮时触发的事件

            await self.update_data()

        elif event.button.id == "button-game_uid-reset":
            # 按下“重置”按钮时触发的事件

            self.reset_selected()
            self.app.notice("已重置游戏账号选择")

    def compose(self) -> ComposeResult:
        yield self.text_view
        with Horizontal():
            yield self.button_select
            yield self.button_refresh
            yield self.button_reset
            yield self.loading
        yield self.option_list


class AddressContent(BaseExchangePlan):
    """
    收货地址选择视图
    """

    DEFAULT_TEXT = Markdown("- 请选择一个收货地址")
    REQUIRE_ACCOUNT_TEXT = Markdown("- 请先完成米游社账号选择")
    UNNEEDED_TEXT = Markdown("- 兑换虚拟商品无需设置收货地址")

    text_view = StaticStatus(REQUIRE_ACCOUNT_TEXT)

    button_select = ControllableButton("💾 保存", id="button-address-select", disabled=True)
    button_refresh = ControllableButton("🔄 刷新", variant="primary", id="button-address-refresh", disabled=True)
    button_reset = ControllableButton("↩ 重置", variant="warning", id="button-address-reset", disabled=True)

    loading = LoadingDisplay()
    loading.hide()

    empty_data_option = Option("暂无收货地址数据 请尝试刷新", disabled=True)
    option_list = OptionList(REQUIRE_ACCOUNT_TEXT, disabled=True)
    """收货地址选项列表"""
    option_list.highlighted = None
    address_list: List[Address] = []
    """收货地址列表"""

    async def update_data(self):
        """
        更新收货地址列表
        """
        if AccountContent._selected is None:
            return

        # 进度条、刷新按钮、选项列表
        self.loading.show()
        self.button_refresh.disable()
        self.option_list.disabled = False

        address_status, AddressContent.address_list = await get_address(AccountContent._selected)
        self.option_list.clear_options()
        if not address_status:
            self.app.notice(f"[bold red]获取收货地址列表失败！[/]")
        if self.address_list:
            self.option_list.add_option(Separator())
            for address_data in self.address_list:
                preview_text = f"[list]" \
                               f"\n👓 收货人：[bold underline]{address_data.connect_name}[/]" \
                               f"\n📞 联系电话：[bold underline]{address_data.phone}[/]" \
                               f"\n📮 收货地址：" \
                               f"\n     省：[bold underline]{address_data.province_name}[/]" \
                               f"\n     市：[bold underline]{address_data.city_name}[/]" \
                               f"\n     区/县：[bold underline]{address_data.county_name}[/]" \
                               f"\n     详细地址：[bold underline]{address_data.addr_ext}[/]" \
                               f"\n📌 地址ID：[bold underline]{address_data.id}[/]" \
                               f"\n[/list]"
                self.option_list.add_option(Option(preview_text))
                self.option_list.add_option(Separator())
                self.option_list.disabled = False
                self.button_select.enable()
        else:
            self.option_list.add_option(self.empty_data_option)

        # 进度条、刷新按钮
        self.loading.hide()
        self.button_refresh.enable()

        #  重置已选地址
        self.reset_selected()

        # 检查选项列表是否为空的操作包含在 check_good_type 中
        self.check_good_type()

    def reset_all(self):
        self.selected = None
        self.text_view.update(self.REQUIRE_ACCOUNT_TEXT)
        self.option_list.disabled = True
        self.option_list.clear_options()
        self.option_list.add_option(self.REQUIRE_ACCOUNT_TEXT)
        self.button_select.disable()
        self.button_reset.disable()
        self.button_refresh.disable()

    @classmethod
    def check_empty(cls):
        """
        检查选项列表是否为空
        """
        if cls.address_list:
            cls.button_select.enable()
            cls.option_list.disabled = False
        else:
            cls.button_select.disable()
            cls.option_list.disabled = True

    @classmethod
    def check_good_type(cls):
        """
        检查商品类型是否是虚拟商品，并改变视图
        """
        # 程序载入初次刷新商品列表时，重置已选商品并调用check_good_type，由于没有选择商品，不需要检查商品类型
        if AccountContent._selected is not None:
            good: Optional[Good] = GoodsContent._selected
            if good is not None and good.is_virtual:
                cls.text_view.update(cls.UNNEEDED_TEXT)
                cls.option_list.disabled = True
                cls.button_select.disable()
                cls.button_refresh.disable()
            elif cls._selected is None:
                cls.text_view.update(cls.DEFAULT_TEXT)
                cls.check_empty()
                cls.button_refresh.enable()
            else:
                # 在已选地址不为空的情况下，视图被虚拟商品改变后的情况
                ExchangePlanView.address_content._set_select_view(cls._selected)

    def reset_selected(self):
        """
        重置已选地址
        """
        self.selected = None
        self.button_reset.disable()
        self.text_view.update(self.DEFAULT_TEXT)
        self.check_good_type()

    def _set_select_view(self, address: Address):
        """
        设置已选地址后改变视图
        """
        self.text_view.update(f"已选择收货地址："
                              f"\n[list]"
                              f"\n📌 地址ID - [bold green]{address.id}[/]"
                              f"\n[/list]")

        # 禁用 选项列表、保存按钮，启用 重置按钮
        self.button_reset.enable()
        self.button_select.disable()
        self.option_list.disabled = True

    async def _on_button_pressed(self, event: ControllableButton.Pressed) -> None:
        if event.button.id == "button-address-select":
            # 按下“保存”按钮时触发的事件

            address_index = self.option_list.highlighted
            if address_index is None:
                self.app.notice("[bold red]请先从列表中选择收货地址！[/]")
                return
            if address_index >= len(self.address_list):
                self.app.notice("[bold red]无法找到收货地址[/]")
                return
            address = self.address_list[address_index]
            self.selected = address

            self._set_select_view(address)

        elif event.button.id == "button-address-refresh":
            # 按下“刷新”按钮时触发的事件

            await self.update_data()

        elif event.button.id == "button-address-reset":
            # 按下“重置”按钮时触发的事件

            self.reset_selected()
            self.app.notice("已重置收货地址选择")

    def compose(self) -> ComposeResult:
        yield self.text_view
        with Horizontal():
            yield self.button_select
            yield self.button_refresh
            yield self.button_reset
            yield self.loading
        yield self.option_list


class CheckOutText(StaticStatus):
    """
    兑换计划预览文本
    """
    DEFAULT_TEXT = "[bold yellow]待选取[/]"
    UNNEEDED_TEXT = "[bold gray]无需设置[/]"
    account_text = reactive(DEFAULT_TEXT)
    address_detail = reactive(DEFAULT_TEXT)
    goods_name = reactive(DEFAULT_TEXT)
    goods_time = reactive(DEFAULT_TEXT)
    game_uid_text = reactive(DEFAULT_TEXT)

    def set_check_item(self,
                       value: Union[UserAccount, Address, Good, None],
                       content_type: Optional[Type[BaseExchangePlan]] = None
                       ):
        """
        传入 Union[UserAccount, Address, Good] 对象，设置对应的文本内容
        :param value: 兑换计划所需的数据对象
        :param content_type: 当 value 为 None 时，需要传入 BaseExchangePlan 对象，用于确定数据类型
        """

        def finished_style_text(text: str):
            return f"[bold green]{text}[/]"

        if value is None:
            if content_type == AccountContent:
                self.account_text = self.DEFAULT_TEXT
            elif content_type == AddressContent:
                good: Optional[Good] = ExchangePlanView.goods_content.selected
                if good is not None and not good.is_virtual:
                    self.address_detail = self.DEFAULT_TEXT
            elif content_type == GoodsContent:
                self.goods_name = self.DEFAULT_TEXT
                self.goods_time = self.DEFAULT_TEXT
                # 把“无需设置”还原为“待选取”
                self.address_detail = self.DEFAULT_TEXT
                self.game_uid_text = self.DEFAULT_TEXT
            elif content_type == GameRecordContent:
                good: Optional[Good] = ExchangePlanView.goods_content.selected
                if good is not None and good.is_virtual:
                    self.game_uid_text = self.DEFAULT_TEXT

            ExchangePlanView.finish_content.button_submit.disable()
            ExchangePlanView.finish_content.button_test.disable()
        else:
            if isinstance(value, UserAccount):
                self.account_text = finished_style_text(value.bbs_uid)
            elif isinstance(value, Address):
                self.address_detail = finished_style_text(value.addr_ext)
            elif isinstance(value, Good):
                self.goods_name = finished_style_text(value.general_name)
                self.goods_time = finished_style_text(value.time_text)
                if value.is_virtual:
                    self.address_detail = self.UNNEEDED_TEXT
                else:
                    self.game_uid_text = self.UNNEEDED_TEXT
            elif isinstance(value, GameRecord):
                self.game_uid_text = finished_style_text(str(value.game_role_id))

            account: UserAccount = ExchangePlanView.account_content.selected
            good: Good = ExchangePlanView.goods_content.selected
            address: Address = ExchangePlanView.address_content.selected
            record: GameRecord = ExchangePlanView.game_record_content.selected
            if account is not None \
                    and good is not None \
                    and (address is not None or good.is_virtual) \
                    and (record is not None or not good.is_virtual):
                ExchangePlanView.finish_content.button_submit.enable()
                ExchangePlanView.finish_content.button_test.enable()
            else:
                ExchangePlanView.finish_content.button_submit.disable()
                ExchangePlanView.finish_content.button_test.disable()

        self.refresh()

    def render(self) -> RenderableType:
        return f"请确认兑换计划信息：" \
               f"\n[list]" \
               f"\n👓 米游社账号 - {self.account_text}" \
               f"\n📦 商品名称 - {self.goods_name}" \
               f"\n📅 兑换时间 - {self.goods_time}" \
               f"\n🎮 游戏UID - {self.game_uid_text}" \
               f"\n📮 收货地址 - {self.address_detail}" \
               f"\n[/list]"


class FinishContent(ExchangePlanContent):
    """
    完成兑换计划添加的视图
    """
    check_out_text = CheckOutText()
    button_submit = ControllableButton("💾 保存兑换计划", variant="success", id="button-finish-submit", disabled=True)
    button_test = ControllableButton("🔧 测试兑换", id="button-finish-test", disabled=True)
    loading = LoadingDisplay()
    loading.hide()

    def compose(self) -> ComposeResult:
        yield self.check_out_text
        with Horizontal():
            yield self.button_submit
            yield self.button_test
            yield self.loading

    async def _on_button_pressed(self, event: ControllableButton.Pressed):
        if event.button.id.startswith("button-finish"):
            account: UserAccount = ExchangePlanView.account_content.selected
            good: Good = ExchangePlanView.goods_content.selected
            address: Optional[Address] = ExchangePlanView.address_content.selected
            record: Optional[GameRecord] = ExchangePlanView.game_record_content.selected
            plan = ExchangePlan(good=good,
                                address=address,
                                account=account,
                                game_record=record)
            if event.button.id == "button-finish-submit":
                if plan in conf.exchange_plans:
                    self.app.notice(f"[bold yellow]该兑换计划已存在[/]")
                else:
                    conf.exchange_plans.add(plan)
                    if not plan.account.device_fp:
                        logger.info(f"账号 {plan.account.bbs_uid} 未设置 device_fp，正在获取...")
                        fp_status, plan.account.device_fp = await get_device_fp(plan.account.device_id_ios)
                        if fp_status:
                            logger.info(f"成功获取 device_fp: {plan.account.device_fp}")
                    if conf.save():
                        self.app.notice(f"[bold green]已保存兑换计划[/]")
                    else:
                        self.app.notice(f"[bold red]保存兑换计划失败[/]")
                        # TODO: 保存失败的具体原因提示
                    await ExchangePlanView.manager_content.update_data()
            elif event.button.id == "button-finish-test":
                exchange_status, _ = await good_exchange(plan)
                if not exchange_status.network_error:
                    self.app.notice(f"[bold green]兑换请求已发送，请查看日志[/]")
                else:
                    self.app.notice(f"[bold red]兑换请求发送失败，请检查网络连接[/]")


class ExchangePlanRow(Container):
    """
    兑换计划行
    """

    def __init__(self, plan: ExchangePlan):
        self.plan = plan
        self.button_delete = PlanButton("🧹 删除计划",
                                        variant="warning",
                                        id=f"button-plan_row-delete-{plan.__hash__()}",
                                        plan=plan)
        self.button_confirm = PlanButton("⚠ 确认删除",
                                         variant="error",
                                         id=f"button-plan_row-confirm-{plan.__hash__()}",
                                         plan=plan)

        self.button_test = PlanButton("🔧 测试兑换",
                                      id=f"button-plan_row-test-{plan.__hash__()}",
                                      plan=plan)
        self.button_cancel = PlanButton("↩ 取消删除",
                                        variant="warning",
                                        id=f"button-plan_row-cancel-{plan.__hash__()}",
                                        plan=plan)
        self.button_confirm.hide()
        self.button_cancel.hide()
        super().__init__()

    def compose(self) -> ComposeResult:
        yield StaticStatus(f"[list]"
                           f"\n👓 米游社账号 - {self.plan.account.bbs_uid}"
                           f"\n📦 商品名称 - {self.plan.good.goods_name}"
                           f"\n📅 兑换时间 - {self.plan.good.time_text}"
                           f"\n🎮 游戏UID - {self.plan.game_record.game_role_id if self.plan.game_record is not None else '无需设置'}"
                           f"\n📮 收货地址 - {self.plan.address.addr_ext if self.plan.address is not None else '无需设置'}"
                           f"\n[/list]")
        with Horizontal():
            yield self.button_delete
            yield self.button_confirm
            yield self.button_test
            yield self.button_cancel

    async def _on_button_pressed(self, event: PlanButton.Pressed):
        if event.button.id.startswith("button-plan_row-delete"):
            self.button_delete.hide()
            self.button_confirm.show()
            self.button_test.hide()
            self.button_cancel.show()

        elif event.button.id.startswith("button-plan_row-confirm"):
            conf.exchange_plans.remove(event.button.plan)
            conf.save()
            self.app.notice(f"[bold red]已删除兑换计划[/]")
            await ExchangePlanView.manager_content.update_data()
            # TODO: 删除计划后不刷新整个列表，而是单独删除该行，但目前测试这种方法无效
            # await ManagerContent.list_view.query(ManagerContent.list_item_id(event.button.plan)).remove()
            # ManagerContent.list_view.index = None
            # ManagerContent.list_view.refresh()

        elif event.button.id.startswith("button-plan_row-test"):
            exchange_status, _ = await good_exchange(event.button.plan)
            if not exchange_status.network_error:
                self.app.notice(f"[bold green]兑换请求已发送，请查看日志[/]")
            else:
                self.app.notice(f"[bold red]兑换请求发送失败，请检查网络连接[/]")

        elif event.button.id.startswith("button-plan_row-cancel"):
            self.button_delete.show()
            self.button_confirm.hide()
            self.button_test.show()
            self.button_cancel.hide()


class ManagerContent(ExchangePlanContent):
    """
    管理兑换计划的视图
    """
    DEFAULT_CSS = """
    ManagerContent UnClickableItem {
        padding: 1;
    }
    """
    button_refresh = ControllableButton("🔄 刷新计划列表", id="button-manager-refresh")
    list_view = ListView(initial_index=None)

    @classmethod
    def list_item_id(cls, plan: ExchangePlan):
        """
        生成兑换计划列表项Widget的ID

        :param plan: 兑换计划
        """
        return f"list_item-plan_row-{plan.__hash__()}"

    @property
    def empty_data_item(self):
        """
        :return: 没有数据时的列表项
        """
        return UnClickableItem(Static("暂无兑换计划数据 请尝试刷新"), disabled=True)

    def compose(self) -> ComposeResult:
        yield self.button_refresh
        yield self.list_view

    async def update_data(self):
        """
        刷新兑换计划列表
        """
        await self.list_view.clear()
        for plan in conf.exchange_plans:
            self.list_view.disabled = False
            await self.list_view.append(
                UnClickableItem(
                    ExchangePlanRow(plan),
                    id=self.list_item_id(plan)
                )
            )
        if not conf.exchange_plans:
            self.list_view.disabled = True
            await self.list_view.append(self.empty_data_item)

    async def _on_button_pressed(self, event: ControllableButton.Pressed):
        if event.button.id == "button-manager-refresh":
            await self.update_data()

    async def _on_mount(self, event: events.Mount) -> None:
        await self.update_data()


class ExchangePlanView(Container):
    """
    添加兑换计划 - 视图
    """
    DEFAULT_CSS = """
    ExchangePlanView {
        height: auto;
        width: 1fr;
        border: round #666;
        padding: 2;
        margin: 1 0;
    }
    """

    account_content = AccountContent()
    goods_content = GoodsContent()
    game_record_content = GameRecordContent()
    address_content = AddressContent()
    finish_content = FinishContent()
    manager_content = ManagerContent()

    def compose(self) -> ComposeResult:
        with TabbedContent():
            with TabPane("➕添加计划"):
                with TabbedContent():
                    with TabPane("1.选择米游社账号"):
                        yield self.account_content
                    with TabPane("2.选择目标商品"):
                        yield self.goods_content
                    with TabPane("3.选择游戏账号"):
                        yield self.game_record_content
                    with TabPane("4.选择收货地址"):
                        yield self.address_content
                    with TabPane("5.完成添加"):
                        yield self.finish_content

            with TabPane("✏️管理计划"):
                yield self.manager_content



import inspect
import time
from abc import abstractmethod
from typing import Optional, Literal, NamedTuple, no_type_check, Union, Dict, Any, TypeVar

from pydantic import BaseModel


class BaseModelWithSetter(BaseModel):
    """
    可以使用@property.setter的BaseModel

    目前pydantic 1.10.7 无法使用@property.setter
    issue: https://github.com/pydantic/pydantic/issues/1577#issuecomment-790506164
    """

    @no_type_check
    def __setattr__(self, name, value):
        try:
            super().__setattr__(name, value)
        except ValueError as e:
            setters = inspect.getmembers(
                self.__class__,
                predicate=lambda x: isinstance(x, property) and x.fset is not None
            )
            for setter_name, func in setters:
                if setter_name == name:
                    object.__setattr__(self, name, value)
                    break
            else:
                raise e


class BaseModelWithUpdate(BaseModel):
    """
    可以使用update方法的BaseModel
    """
    _T = TypeVar("_T", bound=BaseModel)

    @abstractmethod
    def update(self, obj: Union[_T, Dict[str, Any]]) -> _T:
        """
        更新数据对象

        :param obj: 新的数据对象或属性字典
        :raise TypeError
        """
        if isinstance(obj, type(self)):
            obj = obj.dict()
        items = filter(lambda x: x[0] in self.__fields__, obj.items())
        for k, v in items:
            setattr(self, k, v)
        return self


class Good(BaseModelWithUpdate):
    """
    商品数据
    """
    type: int
    """为 1 时商品只有在指定时间开放兑换；为 0 时商品任何时间均可兑换"""
    next_time: Optional[int]
    """为 0 表示任何时间均可兑换或兑换已结束"""
    status: Optional[Literal["online", "not_in_sell"]]
    sale_start_time: Optional[str]
    time_by_detail: Optional[int]
    next_num: Optional[int]
    account_exchange_num: int
    """已经兑换次数"""
    account_cycle_limit: int
    """最多可兑换次数"""
    account_cycle_type: str
    """限购类型 Literal["forever", "month", "not_limit"]"""
    game_biz: Optional[str]
    """商品对应的游戏区服（如 hk4e_cn）（单独查询一个商品时）"""
    game: Optional[str]
    """商品对应的游戏"""
    unlimit: Optional[bool]
    """是否为不限量商品"""

    # 以下为实际会用到的属性

    name: Optional[str]
    """商品名称（单独查询一个商品时）"""
    goods_name: Optional[str]
    """商品名称（查询商品列表时）"""

    goods_id: str
    """商品ID(Good_ID)"""

    price: int
    """商品价格"""

    icon: str
    """商品图片链接"""

    def update(self, obj: Union["Good", Dict[str, Any]]) -> "Good":
        """
        更新商品信息

        :param obj: 新的商品数据
        :raise TypeError
        """
        return super().update(obj)

    @property
    def time(self):
        """
        兑换时间

        :return:
        如果返回`None`，说明任何时间均可兑换或兑换已结束。
        如果返回`0`，说明该商品需要调用获取详细信息的API才能获取兑换时间
        """
        # "next_time" 为 0 表示任何时间均可兑换或兑换已结束
        if self.next_time == 0:
            return None
        elif self.status == "not_in_sell" and self.sale_start_time is not None:
            return int(self.sale_start_time)
        elif self.status == "online":
            return self.next_time
        else:
            return 0

    @property
    def time_text(self):
        """
        商品的兑换时间文本

        :return:
        如果返回`None`，说明需要进一步查询商品详细信息才能获取兑换时间
        """
        if self.time_end:
            return "已结束"
        elif self.time == 0:
            return None
        elif self.time_limited:
            return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.time))
        else:
            return "任何时间"

    @property
    def stoke_text(self):
        """
        商品的库存文本
        """
        if self.time_end:
            return "无"
        elif self.time_limited:
            return str(self.num)
        else:
            return "不限"

    @property
    def time_limited(self):
        """
        是否为限时商品
        """
        # 不限量被认为是不限时商品
        return not self.unlimit

    @property
    def time_end(self):
        """
        兑换是否已经结束
        """
        return self.next_time == 0

    @property
    def num(self):
        """
        库存
        如果返回`None`，说明库存不限
        """
        if self.type != 1 and self.next_num == 0:
            return None
        else:
            return self.next_num

    @property
    def limit(self):
        """
        限购，返回元组 (已经兑换次数, 最多可兑换次数, 限购类型)
        """
        return (self.account_exchange_num,
                self.account_cycle_limit, self.account_cycle_type)

    @property
    def is_virtual(self):
        """
        是否为虚拟商品
        """
        return self.type == 2

    @property
    def general_name(self):
        return self.name or self.goods_name


class GameRecord(BaseModel):
    """
    用户游戏数据
    """
    region_name: str
    """服务器区名"""

    game_id: int
    """游戏ID"""

    level: int
    """用户游戏等级"""

    region: str
    """服务器区号"""

    game_role_id: str
    """用户游戏UID"""

    nickname: str
    """用户游戏昵称"""


class GameInfo(BaseModel):
    """
    游戏信息数据
    """
    # ABBR_TO_ID: Dict[int, Tuple[str, str]] = {}
    # '''
    # 游戏ID(game_id)与缩写和全称的对应关系
    # >>> {游戏ID, (缩写, 全称)}
    # '''
    id: int
    """游戏ID"""

    app_icon: str
    """游戏App图标链接(大)"""

    op_name: str
    """游戏代号(英文数字, 例如hk4e)"""

    en_name: str
    """游戏代号2(英文数字, 例如ys)"""

    icon: str
    """游戏图标链接(圆形, 小)"""

    name: str
    """游戏名称"""


class Address(BaseModel):
    """
    地址数据
    """
    connect_areacode: str
    """电话区号"""
    connect_mobile: str
    """电话号码"""

    # 以下为实际会用到的属性

    province_name: str
    """省"""

    city_name: str
    """市"""

    county_name: str
    """区/县"""

    addr_ext: str
    """详细地址"""

    connect_name: str
    """收货人姓名"""

    id: str
    """地址ID"""

    @property
    def phone(self) -> str:
        """
        联系电话(包含区号)
        """
        return self.connect_areacode + " " + self.connect_mobile


class MmtData(BaseModel):
    """
    短信验证码-人机验证任务申请-返回数据
    """
    challenge: Optional[str]
    gt: str
    mmt_key: str
    new_captcha: bool
    risk_type: Optional[str]
    """任务类型，如滑动拼图 slide"""
    success: Optional[int]
    use_v4: Optional[bool]
    """是否使用极验第四代 GT4"""


class BaseApiStatus(BaseModel):
    """
    API返回结果基类
    """
    success = False
    """成功"""
    network_error = False
    """连接失败"""
    incorrect_return = False
    """服务器返回数据不正确"""
    login_expired = False
    """登录失效"""
    need_verify = False
    """需要进行人机验证"""
    invalid_ds = False
    """Headers DS无效"""

    def __bool__(self):
        if self.success:
            return True
        else:
            return False


class CreateMobileCaptchaStatus(BaseApiStatus):
    """
    发送短信验证码 返回结果
    """
    incorrect_geetest = False
    """人机验证结果数据无效"""


class GetCookieStatus(BaseApiStatus):
    """
    获取Cookie 返回结果
    """
    incorrect_captcha = False
    """验证码错误"""
    missing_login_ticket = False
    """Cookies 缺少 login_ticket"""
    missing_bbs_uid = False
    """Cookies 缺少 bbs_uid (stuid, ltuid, ...)"""
    missing_cookie_token = False
    """Cookies 缺少 cookie_token"""
    missing_stoken = False
    """Cookies 缺少 stoken"""
    missing_stoken_v1 = False
    """Cookies 缺少 stoken_v1"""
    missing_stoken_v2 = False
    """Cookies 缺少 stoken_v2"""
    missing_mid = False
    """Cookies 缺少 mid"""


class GetGoodDetailStatus(BaseApiStatus):
    """
    获取商品详细信息 返回结果
    """
    good_not_existed = False


class ExchangeStatus(BaseApiStatus):
    """
    兑换操作 返回结果
    """
    missing_stoken = False
    """商品为游戏内物品，但 Cookies 缺少 stoken"""
    missing_mid = False
    """商品为游戏内物品，但 stoken 为 'v2' 类型同时 Cookies 缺少 mid"""
    missing_address = False
    """商品为实体物品，但未配置收货地址"""
    missing_game_uid = False
    """商品为游戏内物品，但未配置对应游戏的账号UID"""
    unsupported_game = False
    """暂不支持兑换对应分区/游戏的商品"""
    failed_getting_game_record = False
    """获取用户 GameRecord 失败"""
    init_required = False
    """未进行兑换任务初始化"""
    account_not_found = False
    """账号不存在"""


class GetFpStatus(BaseApiStatus):
    """
    兑换操作 返回结果
    """
    invalid_arguments = False
    """参数错误"""


GeetestResult = NamedTuple("GeetestResult", validate=str, seccode=str)
"""人机验证结果数据"""


class GeetestResultV4(BaseModel):
    """
    GEETEST GT4 人机验证结果数据
    """
    captcha_id: str
    lot_number: str
    pass_token: str
    gen_time: str
    captcha_output: str
